%Octave Script to control PNA to measure time domain sVSWR
%this code works with Gnu Octave
%
% instrument-control toolbox is required
% to load instrument-control if it is not loaded in Octave
% pkg load instrument-control
%
% user need to setup the analyzer including IFBW, frequencies, S21,
% calibrations, number of points, source power etc. 
%



vna=tcp("127.0.0.1",5025,100)

tcp_write(vna,"*IDN?\n");
data = tcp_read(vna, 100);
% Convert uint8 array to string, 
char(data)


GATESPAN=4.5e-9; %time gate span in s
T1=-10e-9;
T2=200e-9;
displaybeta=6;

%quietzone radius

while(1)  %get test location L1...L6/R1...R6/C1...C6/F1...F6
    testlocation=input('input test location (must start with F1...F6,C1...C6 etc) =','s');
    testlocation=upper(testlocation);
    switch(testlocation(1))
        case{'L','l','R','r'}
        dref=sqrt((dnominal+radius)^2+radius^2)-0.4;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        case{'C','c'}
        dref=dnominal+radius;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        case{'f','F'}
        dref=dnominal;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        case{'B','b'}
        dref=dnominal+2*radius;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        otherwise
        display('Not a valid input');
        end;
end

tcp_write(vna,"SENSe1:SWEep:POIN?\n"); % number of points
numpts=str2num(char(tcp_read(vna,36)));

tcp_write(vna,"SENS:FREQ:START?\n");
freqstart=str2num(char(tcp_read(vna,36)));

tcp_write(vna,"SENS:FREQ:STOP?\n");
freqend=str2num(char(tcp_read(vna,36)));
freqs=freqstart+([1:numpts]-1).*(freqend-freqstart)./(numpts-1);  %all freqs
freq_del=(freqend - freqstart)/(numpts-1);
buffersize=40*numpts+500;

%tcp_write(vna,"CALC:PAR:SEL CH1_S11_1\n")
tcp_write(vna,"CALC1:PAR:MNUM:SEL 1\n")
tcp_write(vna,"CALC:TRANSFORM:TIME:STATE OFF\n"); % freq domain
tcp_write(vna,"CALC:FILTer:GATE:TIME:STATE OFF\n"); %turns off time gating
tcp_write(vna,"CALC:FORM MLOG\n"); %log magnitude

%trigger a measurement and wait for it to finish
%fprintf(vna,'ABORT; :INITIATE:IMMEDIATE');  %restart the measurement
tcp_write(vna,"ABORT\n");
%fprintf(vna,'*WAI');
display('wait for sweep to finish.  Press key to continue...');
kbhit; %pause;

% Non-gated data

tcp_write(vna,"FORM ASCII\n");

tcp_write(vna,"CALC:FORM MLOG\n"); %log magnitude
tcp_write(vna,"CALCulate:DATA? FDATA\n"); % get data
temp=char(tcp_read(vna, buffersize));
mag_gnot=str2num(temp);
%FDATA - Formatted Meas, after gating
%FMEM - Formatted memory, after gating
%SDATA - Corrected, Complex Meas before gating
%SMEM - Corrected, Complex Memory before gating

%catch up on buffer
temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
display(['OPC=',char(temp)])

%pause(1);
tcp_write(vna,"CALC:FORM PHASE\n");
pause(0.5)
tcp_write(vna,"CALC:DATA? FDATA\n");
temp=char(tcp_read(vna,buffersize));
phase_gnot=str2num(temp);

s210=10.^(mag_gnot/20).*exp(1i.*phase_gnot*pi()/180);
s210=s210.';

%catch up on buffer
temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
display(['OPC=',char(temp)])

%%
%time domain gating

tcp_write(vna,"CALC:FORM MLOG\n"); %log magnitude
tcp_write(vna,"CALC:TRANSFORM:TIME:STATE ON\n"); % turns on time domain
tcp_write(vna,"CALC:FILTer:GATE:TIME:STATE OFF\n"); %turns off gate
tcp_write(vna,["CALC:TRANSFORM:TIME:START ",num2str(T1),"\n"]); %start/stop time
tcp_write(vna,["CALC:TRANSFORM:TIME:STOP ",num2str(T2),"\n"]);

tcp_write(vna,"CALC:MARKER1:STATE ON\n"); %turns on marker
tcp_write(vna,"CALC:MARK:FUNC:TRAC ON\n"); % turns on tracking so max is read for every sweep
tcp_write(vna,"CALC:MARKER1:FUNC:SELECT MAX\n"); %find max for marker

temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
%char(temp)

tcp_write(vna,"CALC:MARK:X?\n"); %read marker position
Tpeak=str2num(char(tcp_read(vna,30)));  %this is peak of the main response

tcp_write(vna,["CALC:FILTER:GATE:TIME:CENTER ",num2str(Tpeak),"\n"]); %set gate center at main response
tcp_write(vna,["CALC:FILTER:GATE:TIME:SPAN ",num2str(GATESPAN),"\n"]); %time gate span

tcp_write(vna,"CALC:FILTER:GATE:TIME:TYPE BPASS\n"); %set bandpass filter,i.e. Gate-in


tcp_write(vna,"CALC:TRANSFORM:TIME:STATE OFF\n"); % return to freq domain
tcp_write(vna,"CALC:FILTer:GATE:TIME:STATE ON\n"); %turns on time gating


% gate_in data
temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
%char(temp)

tcp_write(vna,"CALC:FORM MLOG\n"); %log magnitude
tcp_write(vna,"CALCulate:DATA? FDATA\n"); % get data
temp=char(tcp_read(vna, buffersize));
mag_gin=str2num(temp);
%FDATA - Formatted Meas, after gating
%FMEM - Formatted memory, after gating
%SDATA - Corrected, Complex Meas before gating
%SMEM - Corrected, Complex Memory before gating

temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
display(['OPC=',char(temp)])


tcp_write(vna,"CALC:FORM PHASE\n");
pause(0.5)
tcp_write(vna,"CALC:DATA? FDATA\n");
temp=char(tcp_read(vna,buffersize));
phase_gin=str2num(temp);

temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
display(['OPC=',char(temp)])

tcp_write(vna,"CALC:FILTER:GATE:TIME:TYPE NOTCH\n"); %set NOTCH filter, Gate-out

temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
display(['OPC=',char(temp)])

tcp_write(vna,"CALC:FORM MLOG\n"); %log magnitude
tcp_write(vna,"CALCulate:DATA? FDATA\n"); % get data
temp=char(tcp_read(vna, buffersize));
mag_gout=str2num(temp);

temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
display(['OPC=',char(temp)])
%FDATA - Formatted Meas, after gating
%FMEM - Formatted memory, after gating
%SDATA - Corrected, Complex Meas before gating
%SMEM - Corrected, Complex Memory before gating

%pause(1);
tcp_write(vna,"CALC:FORM PHASE\n");
pause(0.5)
tcp_write(vna,"CALC:DATA? FDATA\n");
temp=char(tcp_read(vna,buffersize));
phase_gout=str2num(temp);

temp=[];
while(isempty(temp))
  tcp_write(vna,"*OPC?\n");
  temp=tcp_read(vna,100);
end 
display(['OPC=',char(temp)])

tcp_write(vna,"CALC:FILTer:GATE:TIME:STATE OFF\n"); %turns off gate
tcp_write(vna,"CALC:FORM MLOG\n"); %log magnitude
%%

%post process, calculating sVSWR
freqTD=freqs;
rhodB=mag_gout-mag_gin;
rho=10.^(rhodB/20);
TDvswrdB=abs(20*log10((1+rho)./(1-rho)));
%specify frequency range of interest (for removing bandedge data).  
WFreqStart=1e9;
WFreqEnd=12e9;

%specify coverage factor for the statistical analysis
kfactor=0.676;

%-------------------------- Moving average/std
freqWindowHz=120e6;  %set moving avg window, e.g., approx 120 MHz
windowsize=freqWindowHz/(freqTD(2)-freqTD(1))+1;
windowsize=2.*round((windowsize+1)/2)-1; %round to nearest odd number
display(['Number of points in the moving window = ',num2str(windowsize)]);

TDwindowsize=windowsize;  %
TDvswrdB=TDvswrdB.';
  TDmavg=movingmean(TDvswrdB,TDwindowsize,[],[]);
  TDstd1=movingstd(TDvswrdB-TDmavg,floor(TDwindowsize/2),'central');
%        movingstd:  windowmode = 'central' --> use a sliding window centered on
%            each point in the series. The window will have total width
%            of 2*k+1 points, thus k points on each side.
display(['TD moving window (Hz) = ',num2str(freqTD(TDwindowsize)-freqTD(1))]);

%------------------------------- plotting all data
figure(1),
plot(freqTD,TDvswrdB,freqTD,(TDmavg+kfactor*TDstd1)',freqTD,6*ones(1,length(freqTD)),'r','LineWidth',2);
axis([WFreqStart WFreqEnd 0 10])
title(['TDsVSWR, position=',testlocation]);
xlabel('frequency (Hz)');
ylabel('sVSWR (dB)');
legend('TD',['TDmavg+',num2str(kfactor),'\sigma']);
grid;

%-------------------------- decimate data @ 50 MHz interval
%[mintemp,index0decimate]=min(abs(freqTD-1e9));
i=1;
for freqTDlist=min(freqTD):50e6:max(freqTD)
    [mintemp,TDDecimateIndex(i)]=min(abs(freqTD-freqTDlist));
    i=i+1;
end
figure(2),
plot(freqTD(TDDecimateIndex),TDvswrdB(TDDecimateIndex),freqTD,(TDmavg+kfactor*TDstd1)',freqTD,6*ones(1,length(freqTD)),'r','LineWidth',2);
%plot(freqTD,(TDmavg+kfactor*TDstd1)',freqTD,6*ones(1,length(freqTD)),'r','LineWidth',2);
axis([WFreqStart WFreqEnd 0 10])
title(['TDsVSWR (50MHz interval), position=',testlocation]);
xlabel('frequency (Hz)');
ylabel('sVSWR (dB)');
legend('TD Decimated',['TDmavg+',num2str(kfactor),'\sigma']);
grid;

dlmwrite(['TD_',testlocation,'.csv'],[freqs',TDvswrdB,TDmavg,TDstd1,TDmavg+kfactor*TDstd1]);
